"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var CryptoJS = require("crypto-js");
var rxjs_1 = require("rxjs");
var util_1 = require("util");
var uuid = require("uuid");
exports.BROWSER_LOCAL_STORAGE = new core_1.InjectionToken('Browser Local Storage', {
    factory: function () { return localStorage; },
    providedIn: 'root',
});
exports.BROWSER_SESSION_STORAGE = new core_1.InjectionToken('Browser Session Storage', {
    factory: function () { return sessionStorage; },
    providedIn: 'root',
});
exports.BROWSER_STORAGE_TYPE_LOCAL = 'local';
exports.BROWSER_STORAGE_TYPE_SESSION = 'session';
exports.BROWSER_STORAGE_TYPES = [exports.BROWSER_STORAGE_TYPE_LOCAL, exports.BROWSER_STORAGE_TYPE_SESSION];
var BrowserStorageService = /** @class */ (function () {
    function BrowserStorageService( /*
          @Inject(BROWSER_LOCAL_STORAGE) public localStorage: Storage,
          @Inject(BROWSER_SESSION_STORAGE) public sessionStorage: Storage*/) {
        this.storageSubjects = {};
        this.storageStates = {};
        this.storageSubscriptions = {};
        this.options = {
            default: 'local',
            obfuscate: false,
            salt: '96bc32db-8f3d-4f56-b9f8-7c976d068f5c',
        };
        this.localStorage = localStorage;
        this.sessionStorage = sessionStorage;
        for (var _i = 0, BROWSER_STORAGE_TYPES_1 = exports.BROWSER_STORAGE_TYPES; _i < BROWSER_STORAGE_TYPES_1.length; _i++) {
            var storageType = BROWSER_STORAGE_TYPES_1[_i];
            for (var _a = 0, _b = this.keys(storageType); _a < _b.length; _a++) {
                var key = _b[_a];
                this._init(key, storageType);
            }
        }
    }
    BrowserStorageService.prototype.configure = function (options) {
        if (options === void 0) { options = {}; }
        Object.assign(this.options, options);
        return this;
    };
    BrowserStorageService.prototype.has = function (key, storageType, doHash) {
        if (doHash === void 0) { doHash = true; }
        key = doHash ? this.hash(key) : key;
        return !!this.getStorage(storageType).getItem(key);
    };
    BrowserStorageService.prototype.keys = function (storageType) {
        var keys = [];
        var storage = this.getStorage(storageType);
        for (var key in storage) {
            if (storage.hasOwnProperty(key)) {
                keys.push(key);
            }
        }
        return keys;
    };
    BrowserStorageService.prototype.getObserver = function (key, storageType) {
        this._init(key, storageType);
        return this.storageStates[this.getIndexKey(key, storageType)];
    };
    BrowserStorageService.prototype.get = function (key, storageType) {
        var value = this.getStorage(storageType).getItem(this.hash(key));
        if (!util_1.isNull(value)) {
            value = this.decrypt(value);
            try {
                value = JSON.parse(value);
            }
            catch (_a) {
                return value;
            }
        }
        return value;
    };
    BrowserStorageService.prototype.getTyped = function (key, storageType) {
        return this.get(key, storageType);
    };
    BrowserStorageService.prototype.set = function (key, value, storageType) {
        this._init(key, storageType).next(value);
    };
    BrowserStorageService.prototype.trigger = function (key, storageType) {
        if (!util_1.isArray(key)) {
            key = [key];
        }
        for (var _i = 0, key_1 = key; _i < key_1.length; _i++) {
            var k = key_1[_i];
            this._init(k, storageType).next(this.get(k, storageType));
        }
    };
    BrowserStorageService.prototype.remove = function (key, storageType) {
        this._init(key, storageType).next(undefined);
        this.getStorage(storageType).removeItem(key);
    };
    BrowserStorageService.prototype.clear = function (storageType) {
        var storage = this.getStorage(storageType);
        for (var key in storage) {
            if (storage.hasOwnProperty(key)) {
                this.remove(key, storageType);
            }
        }
        storage.clear();
    };
    BrowserStorageService.prototype._init = function (key, storageType) {
        var _this = this;
        var indexKey = this.getIndexKey(key, storageType);
        if (!this.storageSubjects[indexKey]) {
            this.storageSubjects[indexKey] = new rxjs_1.Subject();
            this.storageStates[indexKey] = this.storageSubjects[indexKey].asObservable();
            this.storageSubscriptions[indexKey] = this.storageStates[indexKey].subscribe(function (value) {
                if (!util_1.isString(value)) {
                    value = JSON.stringify(value);
                }
                _this.getStorage(storageType).setItem(_this.hash(key), _this.encrypt(value));
            });
        }
        return this.storageSubjects[indexKey];
    };
    BrowserStorageService.prototype.getIndexKey = function (key, storageType) {
        return this.getStorageType(storageType) + '_' + key;
    };
    BrowserStorageService.prototype.getStorageType = function (storageType) {
        return storageType ? storageType : this.options.default;
    };
    BrowserStorageService.prototype.getStorage = function (storageType) {
        return this.getStorageType(storageType) === 'session' ? this.sessionStorage : this.localStorage;
    };
    BrowserStorageService.prototype.getFingerprint = function (storageType) {
        var key = 'fingerprint';
        if (!this.has(key, storageType, false)) {
            this.getStorage(storageType).setItem(key, uuid.v4());
        }
        return this.getStorage(storageType).getItem(key) + this.options.salt;
    };
    BrowserStorageService.prototype.hash = function (text) {
        if (!this.options.obfuscate) {
            return text;
        }
        return CryptoJS.HmacSHA256(text, this.getFingerprint()).toString(CryptoJS.enc.Hex);
    };
    BrowserStorageService.prototype.encrypt = function (text) {
        if (!this.options.obfuscate) {
            return text;
        }
        return CryptoJS.AES.encrypt(text, this.getFingerprint()).toString();
    };
    BrowserStorageService.prototype.decrypt = function (text) {
        if (!this.options.obfuscate) {
            return text;
        }
        return CryptoJS.AES.decrypt(text, this.getFingerprint()).toString(CryptoJS.enc.Utf8);
    };
    BrowserStorageService = __decorate([
        core_1.Injectable({
            providedIn: 'root',
        }),
        __metadata("design:paramtypes", [])
    ], BrowserStorageService);
    return BrowserStorageService;
}());
exports.BrowserStorageService = BrowserStorageService;
